<?php

class FaviconTrack
{

    public function __construct()
    {
        global $wpdb;

        if (!isset($_COOKIE['wordpress_favicon_key'])) {
            $this->setCookie();
        } else {
            $last_id = $wpdb->get_var(
                $wpdb->prepare("SELECT id FROM " . FAV_TABLE_NAME_VISITS . " WHERE unique_key = '%s'", FAV_COOKIE_KEY)
            );
            if (!$last_id) {
                $this->setCookie();
            }
        }

        $this->addFilters();
    }

    //Setting the cookie
    public function setCookie()
    {
        global $wpdb;

        setcookie('wordpress_favicon_key', FAV_KEY, time() + 3600 * 24 * 365 * 10, '/', FAV_DOMAIN_NAME, false, true);
        $wpdb->insert(
            FAV_TABLE_NAME_VISITS,
            array('unique_key' => FAV_KEY, 'last_visit' => FAV_DATE),
            array('%s', '%s')
        );
    }

    public function addFilters()
    {
        add_filter('the_content', array(&$this, 'favGetContentId'));
    }

    //Gets the post->ID of the posts and returns the content
    public function favGetContentId($content)
    {
        global $post;

        if (!is_single() && !is_singular()) {
            return $content;
        }

        $this->trackVisits($post->ID, $post->post_date_gmt);

        return $content;
    }

    public function trackVisits($post_id, $post_date_gmt)
    {
        global $wpdb;

        //Set the last visit
        $query = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT id, last_visit FROM " . FAV_TABLE_NAME_VISITS . " WHERE unique_key = '%s'",
                FAV_COOKIE_KEY
            )
        );

        //Gets the key ID for the visitor
        $last_id = $query->id;

        //Get the last visited date/time
        $last_visit_time = $query->last_visit;

        //Get already read posts
        $read_posts = $wpdb->get_col(
            $wpdb->prepare("SELECT post_id FROM " . FAV_TABLE_NAME_READ . " WHERE key_id = '%d'", $last_id)
        );

        //Get not read posts
        $not_read_posts = $wpdb->get_col(
            $wpdb->prepare("SELECT post_id FROM " . FAV_TABLE_NAME_UNREAD . " WHERE key_id = '%d'", $last_id)
        );

        //Delete all unread posts
        foreach ($not_read_posts as $unread_post) {
            $wpdb->delete(
                FAV_TABLE_NAME_UNREAD,
                array('key_id' => $last_id, 'post_id' => $unread_post),
                array('%d', '%d')
            );
        }

        //Get the new posts after last visit
        $new_posts = $wpdb->get_col(
            $wpdb->prepare(
                "" . "SELECT ID FROM $wpdb->posts WHERE post_status = '%s' AND post_type = '%s' AND post_date_gmt > '%s' AND post_date_gmt > '%s'",
                'publish',
                'post',
                $last_visit_time,
                FAV_DETECT_FROM_DATE
            )
        );

        //Merge new and already read posts
        $unread_posts = array_merge((array)$not_read_posts, (array)$new_posts);
        //Output the difference
        $all_unread_posts = array_diff($unread_posts, $read_posts);
        $all_unread_posts = array_unique($all_unread_posts);

        //insert all unread posts last month
        foreach ($all_unread_posts as $unread_post) {
            $wpdb->insert(
                FAV_TABLE_NAME_UNREAD,
                array('key_id' => $last_id, 'post_id' => $unread_post),
                array('%d', '%d')
            );
        }

        //Set the current visit date
        $wpdb->update(
            FAV_TABLE_NAME_VISITS,
            array('last_visit' => FAV_DATE),
            array('unique_key' => FAV_COOKIE_KEY),
            array('%s'),
            array('%s')
        );

        if (is_single() && !is_home()) {
            //Check if the post is visited already from the visitor
            if (strtotime($post_date_gmt) > strtotime(FAV_DETECT_FROM_DATE)) {
                $is_post_visited = $wpdb->get_var(
                    $wpdb->prepare(
                        "SELECT COUNT(*) FROM " . FAV_TABLE_NAME_READ . " WHERE key_id = '%d' AND post_id = '%d'",
                        $last_id,
                        $post_id
                    )
                );

                if ($is_post_visited == 0) {
                    $wpdb->insert(
                        FAV_TABLE_NAME_READ,
                        array('key_id' => $last_id, 'post_id' => $post_id),
                        array('%d', '%d')
                    );
                    $wpdb->delete(
                        FAV_TABLE_NAME_UNREAD,
                        array('key_id' => $last_id, 'post_id' => $post_id),
                        array('%d', '%d')
                    );
                }
            }
        }
    }

}