<?php
class FaviconSettings
{
    /**
     * Holds the values to be used in the fields callbacks
     */
    private $options;

    /**
     * Start up
     */
    public function __construct()
    {
        add_action('admin_menu', array($this, 'addPluginPage'));
        add_action('admin_init', array($this, 'pageInit'));
    }

    /**
     * Add options page
     */
    public function addPluginPage()
    {
        // This page will be under "Settings"
        add_options_page(
            'Favicon Settings',
            'Favicon',
            'manage_options',
            'favicon_settings',
            array(&$this, 'createAdminPage')
        );
    }

    /**
     * Options page callback
     */
    public function createAdminPage()
    {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }

        // Set class property
        $this->options = get_option('fav_option_name');
        ?>
        <div class="wrap">
            <?php screen_icon(); ?>
            <h2>Favicon Settings</h2>

            <form method="post" action="options.php">
                <?php
                // This prints out all hidden setting fields
                settings_fields('fav_option_group');
                do_settings_sections('fav_setting_admin');
                submit_button();
                ?>
            </form>
        </div>
    <?php
    }

    /**
     * Register and add settings
     */
    public function pageInit()
    {
        register_setting(
            'fav_option_group', // Option group
            'fav_option_name', // Option name
            array($this, 'sanitize') // Sanitize
        );

        add_settings_section(
            'settings_section', // ID
            null, // Title
            null, // Callback
            'fav_setting_admin' // Page
        );

        add_settings_field(
            'fav_animation', // ID
            '' . '<label for="fav_animation">Badge animation type</label>', // Title
            array($this, 'favAnimationCallback'), // Callback
            'fav_setting_admin', // Page
            'settings_section' // Section
        );

        add_settings_field(
            'fav_position',
            '' . '<label for="fav_position">Badge position</label>', // Title
            array($this, 'favPositionCallback'),
            'fav_setting_admin',
            'settings_section'
        );

        add_settings_field(
            'fav_shape',
            '' . '<label for="fav_shape">Badge shape</label>', // Title
            array($this, 'favShapeCallback'),
            'fav_setting_admin',
            'settings_section'
        );

        add_settings_field(
            'fav_bg_color',
            '' . '<label for="fav_bg_color">Badge background color</label>', // Title
            array($this, 'favBgColorCallback'),
            'fav_setting_admin',
            'settings_section'
        );

        add_settings_field(
            'fav_text_color',
            '' . '<label for="fav_text_color">Badge text color</label>', // Title
            array($this, 'favTextColorCallback'),
            'fav_setting_admin',
            'settings_section'
        );

        add_settings_field(
            'fav_badge_update',
            '' . '<label for="fav_badge_update">Badge update interval</label>', // Title
            array($this, 'favBadgeUpdateCallback'),
            'fav_setting_admin',
            'settings_section'
        );

        add_settings_field(
            'fav_posts_period',
            '' . '<label for="fav_posts_period">Badge count posts since</label>', // Title
            array($this, 'favPostsPeriodCallback'),
            'fav_setting_admin',
            'settings_section'
        );

    }

    /**
     * Sanitize each setting field as needed
     *
     * @param array $input Contains all settings fields as array keys
     *
     * @return array
     */
    public function sanitize($input)
    {
        $new_input = array();

        if (isset($input['fav_animation'])) {
            $new_input['fav_animation'] = trim(sanitize_text_field($input['fav_animation']));
        }

        if (isset($input['fav_position'])) {
            $new_input['fav_position'] = trim(sanitize_text_field($input['fav_position']));
        }

        if (isset($input['fav_shape'])) {
            $new_input['fav_shape'] = trim(sanitize_text_field($input['fav_shape']));
        }

        if (isset($input['fav_bg_color'])) {
            $new_input['fav_bg_color'] = '';
            if (preg_match('/^#?+[0-9a-f]{3}(?:[0-9a-f]{3})?$/i', strtolower($input['fav_bg_color']))) {
                $new_input['fav_bg_color'] = trim(sanitize_text_field($input['fav_bg_color']));
            }
        }

        if (isset($input['fav_text_color'])) {
            $new_input['fav_text_color'] = '';
            if (preg_match('/^#?+[0-9a-f]{3}(?:[0-9a-f]{3})?$/i', strtolower($input['fav_text_color']))) {
                $new_input['fav_text_color'] = trim(sanitize_text_field($input['fav_text_color']));
            }
        }

        if (isset($input['fav_badge_update'])) {
            $new_input['fav_badge_update'] = absint(trim($input['fav_badge_update']));
        }

        if (isset($input['fav_posts_period'])) {
            $new_input['fav_posts_period'] = absint(trim($input['fav_posts_period']));
        }

        return $new_input;
    }

    /**
     * Get the settings option array and print one of its values
     */
    public function favAnimationCallback()
    {
        printf(
            '' . '<select id="fav_animation" name="fav_option_name[fav_animation]">' . '<option value="none" ' . ((isset($this->options["fav_animation"]) && $this->options["fav_animation"] == "none") ? "selected='selected'" : "") . '>Without animation</option>' . '<option value="slide" ' . ((isset($this->options["fav_animation"]) && $this->options["fav_animation"] == "slide") ? "selected='selected'" : "") . '>Slide animation</option>' . '<option value="fade" ' . ((isset($this->options["fav_animation"]) && $this->options["fav_animation"] == "fade") ? "selected='selected'" : "") . '>Fade animation</option>' . '<option value="pop" ' . ((isset($this->options["fav_animation"]) && $this->options["fav_animation"] == "pop") ? "selected='selected'" : "") . '>Pop animation</option>' . '<option value="popFade" ' . ((isset($this->options["fav_animation"]) && $this->options["fav_animation"] == "popFade") ? "selected='selected'" : "") . '>Pop & fade animation</option>' . '</select>'
        );
    }

    /**
     * Get the settings option array and print one of its values
     */
    public function favPositionCallback()
    {
        printf(
            '' . '<select id="fav_position" name="fav_option_name[fav_position]">' . '<option value="up" ' . ((isset($this->options["fav_position"]) && $this->options["fav_position"] == "up") ? "selected='selected'" : "") . '>Top</option>' . '<option value="down" ' . ((isset($this->options["fav_position"]) && $this->options["fav_position"] == "down") ? "selected='selected'" : "") . '>Bottom</option>' . '<option value="left" ' . ((isset($this->options["fav_position"]) && $this->options["fav_position"] == "left") ? "selected='selected'" : "") . '>Left</option>' . '<option value="upleft" ' . ((isset($this->options["fav_position"]) && $this->options["fav_position"] == "upleft") ? "selected='selected'" : "") . '>Top Left</option>' . '</select>'
        );
    }

    public function favShapeCallback()
    {
        printf(
            '' . '<select id="fav_shape" name="fav_option_name[fav_shape]">' . '<option value="circle" ' . ((isset($this->options["fav_shape"]) && $this->options["fav_shape"] == "circle") ? "selected='selected'" : "") . '>Circle</option>' . '<option value="rectangle" ' . ((isset($this->options["fav_shape"]) && $this->options["fav_shape"] == "rectangle") ? "selected='selected'" : "") . '>Rectangle</option>' . '</select>'
        );
    }

    public function favBgColorCallback()
    {
        printf(
            '' . '<input type="text" name="fav_option_name[fav_bg_color]" id="fav_bg_color" class="fav_color" value="' . (isset($this->options["fav_bg_color"]) ? $this->options["fav_bg_color"] : "") . '">'
        );
    }

    public function favTextColorCallback()
    {
        printf(
            '' . '<input type="text" name="fav_option_name[fav_text_color]" id="fav_text_color" class="fav_color" value="' . (isset($this->options["fav_text_color"]) ? $this->options["fav_text_color"] : "") . '">'
        );
    }

    public function favBadgeUpdateCallback()
    {
        printf(
            '' . '<input type="text" name="fav_option_name[fav_badge_update]" id="fav_badge_update" value="' . (isset($this->options["fav_badge_update"]) ? $this->options["fav_badge_update"] : "") . '"> minutes'
        );
    }

    public function favPostsPeriodCallback()
    {
        printf(
            '' . '<input type="text" name="fav_option_name[fav_posts_period]" id="fav_posts_period" value="' . (isset($this->options["fav_posts_period"]) ? $this->options["fav_posts_period"] : "") . '"> days ago'
        );
    }
}